@@ -13,6 +13,7 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
+import copy
 from io import StringIO
 import os
 import sys
@@ -156,25 +157,67 @@ class TestShowVnfLcm(TestVnfLcm):
 
 class TestListVnfLcm(TestVnfLcm):
 
-    vnf_instances = vnflcm_fakes.create_vnf_instances(count=3)
-
     def setUp(self):
         super(TestListVnfLcm, self).setUp()
         self.list_vnf_instance = vnflcm.ListVnfLcm(
             self.app, self.app_args, cmd_name='vnflcm list')
 
     def test_take_action(self):
+        vnf_instances = vnflcm_fakes.create_vnf_instances(count=3)
         parsed_args = self.check_parser(self.list_vnf_instance, [], [])
         self.requests_mock.register_uri(
             'GET', os.path.join(self.url, 'vnflcm/v1/vnf_instances'),
-            json=self.vnf_instances, headers=self.header)
+            json=vnf_instances, headers=self.header)
         actual_columns, data = self.list_vnf_instance.take_action(parsed_args)
 
         headers, columns = tacker_osc_utils.get_column_definitions(
             vnflcm._attr_map, long_listing=True)
 
         expected_data = []
-        for vnf_instance_obj in self.vnf_instances:
+        for vnf_instance_obj in vnf_instances:
+            expected_data.append(vnflcm_fakes.get_vnflcm_data(
+                vnf_instance_obj, columns=columns, list_action=True))
+
+        self.assertCountEqual(_get_columns_vnflcm(action='list'),
+                              actual_columns)
+        self.assertCountEqual(expected_data, list(data))
+
+    def test_take_action_with_pagination(self):
+        vnf_instances = vnflcm_fakes.create_vnf_instances(count=3)
+        next_links_num = 3
+        parsed_args = self.check_parser(self.list_vnf_instance, [], [])
+        path = os.path.join(self.url, 'vnflcm/v1/vnf_instances')
+
+        links = [0] * next_links_num
+        link_headers = [0] * next_links_num
+
+        for i in range(next_links_num):
+            links[i] = (
+                '{base_url}?nextpage_opaque_marker={vnf_instance_id}'.format(
+                    base_url=path,
+                    vnf_instance_id=vnf_instances[i]['id']))
+            link_headers[i] = copy.deepcopy(self.header)
+            link_headers[i]['Link'] = '<{link_url}>; rel="next"'.format(
+                link_url=links[i])
+
+        self.requests_mock.register_uri(
+            'GET', path, json=[vnf_instances[0]], headers=link_headers[0])
+        self.requests_mock.register_uri(
+            'GET', links[0], json=[vnf_instances[1]],
+            headers=link_headers[1])
+        self.requests_mock.register_uri(
+            'GET', links[1], json=[vnf_instances[2]],
+            headers=link_headers[2])
+        self.requests_mock.register_uri(
+            'GET', links[2], json=[], headers=self.header)
+
+        actual_columns, data = self.list_vnf_instance.take_action(parsed_args)
+
+        headers, columns = tacker_osc_utils.get_column_definitions(
+            vnflcm._attr_map, long_listing=True)
+
+        expected_data = []
+        for vnf_instance_obj in vnf_instances:
             expected_data.append(vnflcm_fakes.get_vnflcm_data(
                 vnf_instance_obj, columns=columns, list_action=True))
 
@@ -254,9 +297,9 @@ class TestInstantiateVnfLcm(TestVnfLcm):
                                self.instantiate_vnf_lcm.take_action,
                                parsed_args)
 
-        expected_msg = ("Invalid input: File %s does not exist "
-                        "or user does not have read privileges to it")
-        self.assertEqual(expected_msg % sample_param_file, str(ex))
+        expected_msg = ("Invalid input: "
+                        "User does not have read privileges to it")
+        self.assertEqual(expected_msg, str(ex))
 
     @mock.patch("os.open")
     @mock.patch("os.access")
@@ -288,6 +331,9 @@ class TestHealVnfLcm(TestVnfLcm):
         self.heal_vnf_lcm = vnflcm.HealVnfLcm(
             self.app, self.app_args, cmd_name='vnflcm heal')
 
+    _heal_sample_param_file = ("./tackerclient/osc/v1/vnflcm/samples/"
+                               "heal_vnf_instance_param_sample.json")
+
     @ddt.data((['--cause', 'test-cause', "--vnfc-instance",
                 'vnfc-id-1', 'vnfc-id-2'],
                [('cause', 'test-cause'),
@@ -296,6 +342,8 @@ class TestHealVnfLcm(TestVnfLcm):
                [('cause', 'test-cause')]),
               (["--vnfc-instance", 'vnfc-id-1', 'vnfc-id-2'],
                [('vnfc_instance', ['vnfc-id-1', 'vnfc-id-2'])]),
+              (["--additional-param-file", _heal_sample_param_file],
+               [('additional_param_file', _heal_sample_param_file)]),
               ([], []))
     @ddt.unpack
     def test_take_action(self, arglist, verifylist):
@@ -340,6 +388,25 @@ class TestHealVnfLcm(TestVnfLcm):
                           self.heal_vnf_lcm.take_action,
                           parsed_args)
 
+    def test_take_action_param_file_not_exists(self):
+        vnf_instance = vnflcm_fakes.vnf_instance_response()
+        sample_param_file = "./not_exists.json"
+        arglist = [vnf_instance['id'],
+                   '--additional-param-file', sample_param_file]
+        verifylist = [('vnf_instance', vnf_instance['id']),
+                      ('additional_param_file', sample_param_file)]
+
+        # command param
+        parsed_args = self.check_parser(self.heal_vnf_lcm, arglist,
+                                        verifylist)
+
+        ex = self.assertRaises(exceptions.InvalidInput,
+                               self.heal_vnf_lcm.take_action, parsed_args)
+
+        expected_msg = ("Invalid input: "
+                        "User does not have read privileges to it")
+        self.assertEqual(expected_msg, str(ex))
+
 
 @ddt.ddt
 class TestTerminateVnfLcm(TestVnfLcm):
@@ -396,7 +463,7 @@ class TestTerminateVnfLcm(TestVnfLcm):
             self.assertIn(expected_message, actual_message)
 
             if delete_vnf:
-                expected_message = ("VNF Instance '%s' deleted successfully"
+                expected_message = ("VNF Instance '%s' is deleted successfully"
                                     % vnf_instance['id'])
                 self.assertIn(expected_message, actual_message)
 
@@ -512,7 +579,7 @@ class TestDeleteVnfLcm(TestVnfLcm):
         sys.stdout = buffer = StringIO()
         result = self.delete_vnf_instance.take_action(parsed_args)
         self.assertIsNone(result)
-        self.assertEqual(("Vnf instance '%s' deleted successfully")
+        self.assertEqual(("Vnf instance '%s' is deleted successfully")
                          % self.vnf_instances[0]['id'],
                          buffer.getvalue().strip())
 
@@ -707,9 +774,9 @@ class TestScaleVnfLcm(TestVnfLcm):
         ex = self.assertRaises(exceptions.InvalidInput,
                                self.scale_vnf_lcm.take_action, parsed_args)
 
-        expected_msg = ("Invalid input: File %s does not exist "
-                        "or user does not have read privileges to it")
-        self.assertEqual(expected_msg % sample_param_file, str(ex))
+        expected_msg = ("Invalid input: "
+                        "User does not have read privileges to it")
+        self.assertEqual(expected_msg, str(ex))
 
     @ddt.data('SCALE_IN', 'SCALE_OUT')
     def test_take_action_vnf_instance_not_found(self, scale_type):
@@ -821,9 +888,9 @@ class TestChangeExtConnVnfLcm(TestVnfLcm):
             self.change_ext_conn_vnf_lcm.take_action,
             parsed_args)
 
-        expected_msg = ("Invalid input: File %s does not exist "
-                        "or user does not have read privileges to it")
-        self.assertEqual(expected_msg % sample_param_file, str(ex))
+        expected_msg = ("Invalid input: "
+                        "User does not have read privileges to it")
+        self.assertEqual(expected_msg, str(ex))
 
     @mock.patch("os.open")
     @mock.patch("os.access")
